﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [Item("EvolutionTrackingOperator", "A base operator which facilitates access to the genealogy graph.")]
  [StorableClass]
  public class EvolutionTrackingOperator<T> : SingleSuccessorOperator where T : class,IItem {
    // evolution tracking-related parameters
    private const string resultsParameterName = "Results";
    private const string populationGraphParameterName = "PopulationGraph";
    private const string generationsParameterName = "Generations";

    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[resultsParameterName]; }
    }
    public ILookupParameter<IntValue> GenerationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters[generationsParameterName]; }
    }
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    public IntValue Generations {
      get { return GenerationsParameter.ActualValue; }
    }
    public IGenealogyGraph<T> GenealogyGraph {
      get {
        IResult result;
        if (!Results.ContainsKey(populationGraphParameterName)) {
          result = new Result(populationGraphParameterName, new GenealogyGraph<T>());
          Results.Add(result);
        } else {
          result = Results[populationGraphParameterName];
        }
        var graph = (GenealogyGraph<T>)result.Value;
        return graph;
      }
    }
    public EvolutionTrackingOperator() {
      Parameters.Add(new LookupParameter<IntValue>(generationsParameterName));
      Parameters.Add(new LookupParameter<ResultCollection>(resultsParameterName));
    }
    protected EvolutionTrackingOperator(EvolutionTrackingOperator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EvolutionTrackingOperator<T>(this, cloner);
    }
    [StorableConstructor]
    protected EvolutionTrackingOperator(bool deserializing) : base(deserializing) { }
  }
}
